package com.diagbot.facade;

import cn.afterturn.easypoi.exception.excel.ExcelImportException;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.diagbot.client.CdssCoreClient;
import com.diagbot.dto.DictionaryInfoDTO;
import com.diagbot.dto.IndexBatchDTO;
import com.diagbot.dto.KllisDetailDTO;
import com.diagbot.dto.LisDetailDTO;
import com.diagbot.dto.RespDTO;
import com.diagbot.entity.AnesthesiaConfig;
import com.diagbot.entity.DeptConfig;
import com.diagbot.entity.DiseaseConfig;
import com.diagbot.entity.DrugConfig;
import com.diagbot.entity.LisConfig;
import com.diagbot.entity.MappingConfig;
import com.diagbot.entity.NurseConfig;
import com.diagbot.entity.OperationConfig;
import com.diagbot.entity.PacsConfig;
import com.diagbot.entity.ScaleConfig;
import com.diagbot.entity.TcmdiseaseConfig;
import com.diagbot.entity.TcmsyndromeConfig;
import com.diagbot.entity.TransfusionConfig;
import com.diagbot.entity.wrapper.MappingConfigWrapper;
import com.diagbot.enums.ConceptTypeEnum;
import com.diagbot.enums.IsDeleteEnum;
import com.diagbot.enums.MatchSourceEnum;
import com.diagbot.exception.CommonErrorCode;
import com.diagbot.exception.CommonException;
import com.diagbot.service.MappingConfigService;
import com.diagbot.service.impl.MappingConfigServiceImpl;
import com.diagbot.util.BeanUtil;
import com.diagbot.util.DateUtil;
import com.diagbot.util.ExcelUtils;
import com.diagbot.util.ListUtil;
import com.diagbot.util.RespDTOUtil;
import com.diagbot.util.StringUtil;
import com.diagbot.util.SysUserUtils;
import com.diagbot.vo.ConceptVO;
import com.diagbot.vo.FilterVO;
import com.diagbot.vo.IdListVO;
import com.diagbot.vo.IdVO;
import com.diagbot.vo.KllisDetailVO;
import com.diagbot.vo.MappingConfigPageVO;
import com.diagbot.vo.MappingConfigVO;
import com.diagbot.vo.RetrievalVO;
import com.google.common.collect.Lists;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.stereotype.Component;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletResponse;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.regex.Pattern;
import java.util.stream.Collectors;

/**
 * @Description:
 * @Author:zhaops
 * @time: 2021/6/9 14:19
 */
@Component
public class MappingConfigFacade extends MappingConfigServiceImpl {

    private static Pattern pattern = Pattern.compile("\\s*|\t|\r|\n");

    @Autowired
    private CdssCoreClient cdssCoreClient;
    @Autowired
    @Qualifier("mappingConfigServiceImpl")
    private MappingConfigService mappingConfigService;
    @Autowired
    private DictionaryFacade dictionaryFacade;

    /**
     * 分页查询
     *
     * @param mappingConfigPageVO
     * @return
     */
    @Override
    public IPage<MappingConfigWrapper> getPage(MappingConfigPageVO mappingConfigPageVO) {
        if (mappingConfigPageVO.getHospitalId() == null) {
            mappingConfigPageVO.setHospitalId(Long.valueOf(SysUserUtils.getCurrentHospitalID()));
        }

        FilterVO filterVO = new FilterVO();
        RespDTO<List<IndexBatchDTO>> respDTO = null;

        //todo 标准词转conceptId
        List<Long> conceptIds = Lists.newLinkedList();
        if (StringUtil.isNotBlank(mappingConfigPageVO.getUniqueName())) {
            filterVO.setInputStr(mappingConfigPageVO.getUniqueName());
            respDTO = cdssCoreClient.filter(filterVO);
            if (RespDTOUtil.respIsOK(respDTO)) {
                if (ListUtil.isNotEmpty(respDTO.data)) {
                    conceptIds.addAll(respDTO.data
                            .stream()
                            .map(IndexBatchDTO::getId)
                            .distinct()
                            .collect(Collectors.toList()));
                } else {
                    conceptIds.add(-1L);
                }
            }
        }
        mappingConfigPageVO.setConceptIds(conceptIds);

        //todo 药品剂型转 conceptId
        List<Long> formConceptIds = Lists.newLinkedList();
        if (StringUtil.isNotBlank(mappingConfigPageVO.getForm())) {
            filterVO.setInputStr(mappingConfigPageVO.getForm());
            respDTO = cdssCoreClient.filter(filterVO);
            if (RespDTOUtil.respIsOK(respDTO)) {
                if (ListUtil.isNotEmpty(respDTO.data)) {
                    formConceptIds.addAll(respDTO.data
                            .stream()
                            .filter(i -> i.getType().equals(ConceptTypeEnum.Form.getKey()))
                            .map(IndexBatchDTO::getId)
                            .distinct()
                            .collect(Collectors.toList()));
                } else {
                    formConceptIds.add(-1L);
                }
            }
        }
        mappingConfigPageVO.setFormConceptIds(formConceptIds);

        //todo 多条匹配
        if (mappingConfigPageVO.getIsMatch() != null && mappingConfigPageVO.getIsMatch().equals(2)) {
            mappingConfigPageVO.setIsMatch(null);
            mappingConfigPageVO.setIsMultiple(1);
        }

        IPage<MappingConfigWrapper> page = super.getPage(mappingConfigPageVO);

        //todo conceptId转标准词
        List<MappingConfigWrapper> records = page.getRecords();
        records = addNames(records);
        page.setRecords(records);

        return page;
    }


    /**
     * 导出文件
     *
     * @param response
     * @param type
     * @param extFileName
     */
    public void exportExcel(HttpServletResponse response, List<?> list, Integer type, String extFileName) {
        String fileName = extFileName + ".xls";
        switch (type) {
            case 1:
                fileName = "检验" + fileName;
                List<LisConfig> lisConfigList = BeanUtil.listCopyTo(list, LisConfig.class);
                ExcelUtils.exportExcel(lisConfigList, null, "sheet1", LisConfig.class, fileName, response, 12.8f);
                break;
            case 3:
                fileName = "检查" + fileName;
                List<PacsConfig> pacsConfigList = BeanUtil.listCopyTo(list, PacsConfig.class);
                ExcelUtils.exportExcel(pacsConfigList, null, "sheet1", PacsConfig.class, fileName, response, 12.8f);
                break;
            case 4:
                fileName = "诊断" + fileName;
                List<DiseaseConfig> diseaseConfigList = BeanUtil.listCopyTo(list, DiseaseConfig.class);
                ExcelUtils.exportExcel(diseaseConfigList, null, "sheet1", DiseaseConfig.class, fileName, response, 12.8f);
                break;
            case 5:
                fileName = "药品" + fileName;
                List<DrugConfig> drugConfigList = BeanUtil.listCopyTo(list, DrugConfig.class);
                ExcelUtils.exportExcel(drugConfigList, getForm(), "sheet1", DrugConfig.class, fileName, response, 12.8f);
                break;
            case 6:
                fileName = "手术和操作" + fileName;
                List<OperationConfig> operationConfigList = BeanUtil.listCopyTo(list, OperationConfig.class);
                ExcelUtils.exportExcel(operationConfigList, null, "sheet1", OperationConfig.class, fileName, response, 12.8f);
                break;
            case 7:
                fileName = "科室" + fileName;
                List<DeptConfig> deptConfigList = BeanUtil.listCopyTo(list, DeptConfig.class);
                ExcelUtils.exportExcel(deptConfigList, null, "sheet1", DeptConfig.class, fileName, response, 12.8f);
                break;
            case 8:
                fileName = "输血" + fileName;
                List<TransfusionConfig> transfusionConfigList = BeanUtil.listCopyTo(list, TransfusionConfig.class);
                ExcelUtils.exportExcel(transfusionConfigList, null, "sheet1", TransfusionConfig.class, fileName, response, 12.8f);
                break;
            case 10:
                fileName = "量表" + fileName;
                List<ScaleConfig> scaleConfigList = BeanUtil.listCopyTo(list, ScaleConfig.class);
                ExcelUtils.exportExcel(scaleConfigList, null, "sheet1", ScaleConfig.class, fileName, response, 12.8f);
                break;
            case 11:
                fileName = "护理" + fileName;
                List<NurseConfig> nurseConfigList = BeanUtil.listCopyTo(list, NurseConfig.class);
                ExcelUtils.exportExcel(nurseConfigList, null, "sheet1", NurseConfig.class, fileName, response, 12.8f);
                break;
            case 12:
                fileName = "中医疾病" + fileName;
                List<TcmdiseaseConfig> tcmdiseaseConfigList = BeanUtil.listCopyTo(list, TcmdiseaseConfig.class);
                ExcelUtils.exportExcel(tcmdiseaseConfigList, null, "sheet1", TcmdiseaseConfig.class, fileName, response, 12.8f);
                break;
            case 13:
                fileName = "中医证候" + fileName;
                List<TcmsyndromeConfig> tcmsyndromeConfigList = BeanUtil.listCopyTo(list, TcmsyndromeConfig.class);
                ExcelUtils.exportExcel(tcmsyndromeConfigList, null, "sheet1", TcmsyndromeConfig.class, fileName, response, 12.8f);
                break;
            case 14:
                fileName = "麻醉" + fileName;
                List<AnesthesiaConfig> anesthesiaConfigList = BeanUtil.listCopyTo(list, AnesthesiaConfig.class);
                ExcelUtils.exportExcel(anesthesiaConfigList, null, "sheet1", AnesthesiaConfig.class, fileName, response, 12.8f);
                break;
            default:
                break;
        }
    }

    /**
     * 剂型说明
     *
     * @return
     */
    private String getForm() {
        String form = "药品模板——药品剂型填写说明[不填";
        //药品剂型
        List<DictionaryInfoDTO> dicTypeMappingList = dictionaryFacade.getListByGroupType(9);
        if (ListUtil.isNotEmpty(dicTypeMappingList)) {
            String formList = dicTypeMappingList.stream()
                    .filter(i -> StringUtil.isNotBlank(i.getName()))
                    .map(i -> i.getName())
                    .distinct()
                    .collect(Collectors.joining("、"));
            if (StringUtil.isNotBlank(formList)) {
                form += "、" + formList;
            }
        }
        form += "]";
        return form;
    }

    /**
     * 导入模板下载
     *
     * @param response
     */
    public void exportExcelModule(HttpServletResponse response, MappingConfigVO mappingConfigVO) {
        exportExcel(response, new ArrayList<>(), mappingConfigVO.getType(), "导入模板");
    }

    /**
     * 数据导出
     *
     * @param response
     */
    public void exportExcel(HttpServletResponse response, MappingConfigPageVO mappingConfigPageVO) {
        if (mappingConfigPageVO.getHospitalId() == null) {
            mappingConfigPageVO.setHospitalId(Long.valueOf(SysUserUtils.getCurrentHospitalID()));
        }
        mappingConfigPageVO.setHisName("");
        mappingConfigPageVO.setHisDetailName("");
        mappingConfigPageVO.setUniqueName("");
        mappingConfigPageVO.setSize(-1L);

        IPage<MappingConfigWrapper> page = this.getPage(mappingConfigPageVO);

        exportExcel(response, page.getRecords(), mappingConfigPageVO.getType(), "映射");
    }

    /**
     * 导出关联关系
     *
     * @param mappingConfigPageVO
     * @return
     */
    public List<MappingConfigWrapper> exportExcel_remote(MappingConfigPageVO mappingConfigPageVO) {
        if (mappingConfigPageVO.getHospitalId() == null) {
            mappingConfigPageVO.setHospitalId(Long.valueOf(SysUserUtils.getCurrentHospitalID()));
        }
        mappingConfigPageVO.setHisName("");
        mappingConfigPageVO.setHisDetailName("");
        mappingConfigPageVO.setUniqueName("");
        mappingConfigPageVO.setSize(-1L);

        IPage<MappingConfigWrapper> page = this.getPage(mappingConfigPageVO);
        return page.getRecords();
    }

    /**
     * 读取导入数据
     *
     * @param file
     * @param type
     * @param source 1-预匹配，2-导入
     * @return
     */
    public List<MappingConfigWrapper> readImportData(MultipartFile file, Integer type, Integer source) {
        List<MappingConfigWrapper> originList = Lists.newLinkedList();
        try {
            switch (type) {
                case 1:
                    List<LisConfig> lisConfigList = ExcelUtils.importExcel(file, 0, 1, LisConfig.class);
                    originList = BeanUtil.listCopyTo(lisConfigList, MappingConfigWrapper.class);
                    break;
                case 3:
                    List<PacsConfig> pacsConfigList = ExcelUtils.importExcel(file, 0, 1, PacsConfig.class);
                    originList = BeanUtil.listCopyTo(pacsConfigList, MappingConfigWrapper.class);
                    break;
                case 4:
                    List<DiseaseConfig> diseaseConfigList = ExcelUtils.importExcel(file, 0, 1, DiseaseConfig.class);
                    originList = BeanUtil.listCopyTo(diseaseConfigList, MappingConfigWrapper.class);
                    break;
                case 5:
                    List<DrugConfig> drugConfigList = ExcelUtils.importExcel(file, 1, 1, DrugConfig.class);
                    originList = BeanUtil.listCopyTo(drugConfigList, MappingConfigWrapper.class);
                    break;
                case 6:
                    List<OperationConfig> operationConfigList = ExcelUtils.importExcel(file, 0, 1, OperationConfig.class);
                    originList = BeanUtil.listCopyTo(operationConfigList, MappingConfigWrapper.class);
                    break;
                case 7:
                    List<DeptConfig> deptConfigList = ExcelUtils.importExcel(file, 0, 1, DeptConfig.class);
                    originList = BeanUtil.listCopyTo(deptConfigList, MappingConfigWrapper.class);
                    break;
                case 8:
                    List<TransfusionConfig> transfusionConfigList = ExcelUtils.importExcel(file, 0, 1, TransfusionConfig.class);
                    originList = BeanUtil.listCopyTo(transfusionConfigList, MappingConfigWrapper.class);
                    break;
                case 10:
                    List<ScaleConfig> scaleConfigList = ExcelUtils.importExcel(file, 0, 1, ScaleConfig.class);
                    originList = BeanUtil.listCopyTo(scaleConfigList, MappingConfigWrapper.class);
                    break;
                case 11:
                    List<NurseConfig> nurseConfigList = ExcelUtils.importExcel(file, 0, 1, NurseConfig.class);
                    originList = BeanUtil.listCopyTo(nurseConfigList, MappingConfigWrapper.class);
                    break;
                case 12:
                    List<TcmdiseaseConfig> tcmdiseaseConfigList = ExcelUtils.importExcel(file, 0, 1, TcmdiseaseConfig.class);
                    originList = BeanUtil.listCopyTo(tcmdiseaseConfigList, MappingConfigWrapper.class);
                    break;
                case 13:
                    List<TcmsyndromeConfig> tcmsyndromeConfigList = ExcelUtils.importExcel(file, 0, 1, TcmsyndromeConfig.class);
                    originList = BeanUtil.listCopyTo(tcmsyndromeConfigList, MappingConfigWrapper.class);
                    break;
                case 14:
                    List<AnesthesiaConfig> anesthesiaConfigList = ExcelUtils.importExcel(file, 0, 1, AnesthesiaConfig.class);
                    originList = BeanUtil.listCopyTo(anesthesiaConfigList, MappingConfigWrapper.class);
                    break;
                default:
                    break;
            }
        } catch (ExcelImportException e) {

        } catch (CommonException e) {
            throw new CommonException(CommonErrorCode.SERVER_IS_ERROR, "校验失败，请使用模板进行数据" + (source == 1 ? "预匹配" : "导入"));
        } catch (Exception e) {

        }
        return originList;
    }

    /**
     * 预匹配导入术语校验
     *
     * @param file
     * @return
     */
    public Boolean dataVerify(MultipartFile file, Integer type) {
        List<MappingConfigWrapper> originList = readImportData(file, type, 1);
        List<MappingConfigWrapper> retList = precDataProcess(originList, type);
        return true;
    }

    /**
     * 导入数据预匹配
     *
     * @param file
     * @param type
     * @param response
     */
    public void precDataMatch(MultipartFile file, Integer type, HttpServletResponse response) {
        List<MappingConfigWrapper> originList = readImportData(file, type, 1);
        List<MappingConfigWrapper> retList = precDataProcess(originList, type);
        exportExcel(response, retList, type, "关联数据（预匹配）");
    }

    /**
     * 远程调用预匹配接口
     *
     * @param file
     * @param type
     * @return
     */
    public List<MappingConfigWrapper> precDataMatch_remote(MultipartFile file, Integer type) {
        List<MappingConfigWrapper> originList = readImportData(file, type, 1);
        List<MappingConfigWrapper> retList = precDataProcess(originList, type);
        return retList;
    }

    /**
     * 预匹配数据处理
     *
     * @param originList
     * @return
     */
    public List<MappingConfigWrapper> precDataProcess(List<MappingConfigWrapper> originList, Integer type) {
        List<MappingConfigWrapper> retList = Lists.newLinkedList();
        List<MappingConfigWrapper> dataList = Lists.newLinkedList();
        List<MappingConfigWrapper> standardList = Lists.newLinkedList();
        List<MappingConfigWrapper> synonymsList = Lists.newLinkedList();
        List<MappingConfigWrapper> codeList = Lists.newLinkedList();
        List<MappingConfigWrapper> historyList = Lists.newLinkedList();
        List<MappingConfigWrapper> drugList = Lists.newLinkedList();

        //去除空行
        /*originList = originList.stream()
                .filter(MappingConfigWrapper::nonNull)
                .collect(Collectors.toList());*/

        //数据完整性校验
        List<Integer> emptyNumList = Lists.newLinkedList();
        Integer rowId = 1;
        for (MappingConfigWrapper item : originList) {
            ++rowId;
            //过滤医院术语名称为空的条目
            if (type.equals(ConceptTypeEnum.LisPack.getKey())) {
                if (StringUtil.isBlank(item.getHisName())
                        && StringUtil.isBlank(item.getHisDetailName())) {
                    continue;
                } else if (StringUtil.isBlank(item.getHisName())
                        && StringUtil.isNotBlank(item.getHisDetailName())) {
                    emptyNumList.add(rowId);
                }
            } else if (type.equals(ConceptTypeEnum.Disease.getKey())
                    || type.equals(ConceptTypeEnum.Operation.getKey())
                    || type.equals(ConceptTypeEnum.Tcmdisease.getKey())
                    || type.equals(ConceptTypeEnum.Tcmsyndrome.getKey())) {
                if (StringUtil.isBlank(item.getHisName())
                        && StringUtil.isBlank(item.getCode())) {
                    continue;
                } else if (StringUtil.isBlank(item.getHisName())
                        && StringUtil.isNotBlank(item.getCode())) {
                    emptyNumList.add(rowId);
                }
            } else {
                if (StringUtil.isNotBlank(item.getHisName())) {
                    item.setHisName(item.getHisName().trim());
                } else {
                    continue;
                }
            }
            if (StringUtil.isNotBlank(item.getHisDetailName())) {
                item.setHisDetailName(item.getHisDetailName().trim());
            } else {
                item.setHisDetailName("");
            }

            if (StringUtil.isNotBlank(item.getHisCode())) {
                item.setHisCode(item.getHisCode().trim());
            } else {
                item.setHisCode("");
            }
            item.setUniqueName("");
            item.setIsMatch(0);
            item.setSource(null);
            dataList.add(item);
        }

        //医院术语名称不允许为空
        if (ListUtil.isNotEmpty(emptyNumList)) {
            throw new CommonException(CommonErrorCode.PARAM_IS_NULL, "预匹配失败，上传文件内行（"
                    + emptyNumList.stream().map(Object::toString)
                    .collect(Collectors.joining("、"))
                    + "）医院术语" + (type.equals(ConceptTypeEnum.LisPack.getKey()) ? "的检验套餐项" : "") + "不可为空。\n");
        }

        if (ListUtil.isEmpty(dataList)) {
            return retList;
        }

        //重复数据去重
        dataList = dataList.stream().distinct().collect(Collectors.toList());
        if (ListUtil.isEmpty(dataList)) {
            return retList;
        }

        //标准词匹配
        List<String> nameList = dataList.stream()
                .filter(i -> StringUtil.isBlank(i.getHisDetailName()))
                .map(MappingConfigWrapper::getHisName)
                .distinct()
                .collect(Collectors.toList());
        List<String> subNameList = dataList.stream()
                .filter(i -> StringUtil.isNotBlank(i.getHisDetailName()))
                .map(MappingConfigWrapper::getHisDetailName)
                .distinct()
                .collect(Collectors.toList());

        ConceptVO conceptVO = new ConceptVO();
        conceptVO.setSource(MatchSourceEnum.StandWord.getKey());
        conceptVO.setType(type);
        conceptVO.setNames(nameList);
        RespDTO<List<IndexBatchDTO>> respDTO = cdssCoreClient.getConceptNames(conceptVO);
        RespDTOUtil.respNGDealCover(respDTO, "标准术语校验失败");
        conceptVO.setType(ConceptTypeEnum.Lis.getKey());
        conceptVO.setNames(subNameList);
        RespDTO<List<IndexBatchDTO>> subRespDTO = cdssCoreClient.getConceptNames(conceptVO);
        RespDTOUtil.respNGDealCover(subRespDTO, "标准术语校验失败");
        Map<String, List<IndexBatchDTO>> nameMap = new HashMap<>();
        Map<String, List<IndexBatchDTO>> subNameMap = new HashMap<>();
        if (ListUtil.isNotEmpty(respDTO.data)) {
            nameMap = respDTO.data.stream().collect(Collectors.groupingBy(IndexBatchDTO::getName));
        }
        if (ListUtil.isNotEmpty(subRespDTO.data)) {
            subNameMap = subRespDTO.data.stream().collect(Collectors.groupingBy(IndexBatchDTO::getName));
        }
        for (MappingConfigWrapper item : dataList) {
            List<IndexBatchDTO> indexList = Lists.newArrayList();
            if (type.equals(ConceptTypeEnum.LisPack.getKey()) && StringUtil.isNotBlank(item.getHisDetailName())) {
                if (subNameMap.containsKey(item.getHisDetailName())) {
                    indexList = subNameMap.get(item.getHisDetailName());
                }
            } else {
                if (nameMap.containsKey(item.getHisName())) {
                    indexList = nameMap.get(item.getHisName());
                }
            }

            if (ListUtil.isEmpty(indexList)) {
                standardList.add(item);
                continue;
            }
            //标准词默认取第一个
            MappingConfigWrapper standardItem = new MappingConfigWrapper();
            BeanUtil.copyProperties(item, standardItem);
            standardItem.setUniqueName(indexList.get(0).getName());
            standardItem.setConceptId(indexList.get(0).getId());
            standardItem.setCode(indexList.get(0).getCode());
            standardItem.setIsMatch(1);
            standardItem.setSource(MatchSourceEnum.StandWord.getKey());
            standardList.add(standardItem);
        }

        //同义词匹配
        List<String> synonymsNameList = standardList.stream()
                .filter(i -> (i.getIsMatch() == null || i.getIsMatch().equals(0))
                        && StringUtil.isBlank(i.getHisDetailName()))
                .map(MappingConfigWrapper::getHisName)
                .distinct()
                .collect(Collectors.toList());
        List<String> synonymsSubNameList = standardList.stream()
                .filter(i -> (i.getIsMatch() == null || i.getIsMatch().equals(0))
                        && StringUtil.isNotBlank(i.getHisDetailName()))
                .map(MappingConfigWrapper::getHisDetailName)
                .distinct()
                .collect(Collectors.toList());
        conceptVO.setSource(MatchSourceEnum.SynonymsWord.getKey());
        conceptVO.setType(type);
        conceptVO.setNames(synonymsNameList);
        respDTO = cdssCoreClient.getConceptNames(conceptVO);
        RespDTOUtil.respNGDealCover(respDTO, "标准术语校验失败");
        conceptVO.setType(ConceptTypeEnum.Lis.getKey());
        conceptVO.setNames(synonymsSubNameList);
        subRespDTO = cdssCoreClient.getConceptNames(conceptVO);
        RespDTOUtil.respNGDealCover(subRespDTO, "标准术语校验失败");
        Map<String, List<IndexBatchDTO>> synonymsNameMap = new HashMap<>();
        Map<String, List<IndexBatchDTO>> synonymsSubNameMap = new HashMap<>();
        if (ListUtil.isNotEmpty(respDTO.data)) {
            synonymsNameMap = respDTO.data.stream().collect(Collectors.groupingBy(IndexBatchDTO::getSynonyms));
        }
        if (ListUtil.isNotEmpty(subRespDTO.data)) {
            synonymsSubNameMap = subRespDTO.data.stream().collect(Collectors.groupingBy(IndexBatchDTO::getSynonyms));
        }
        for (MappingConfigWrapper item : standardList) {
            if (item.getIsMatch() != null && item.getIsMatch().equals(1)) {
                synonymsList.add(item);
                continue;
            }
            List<IndexBatchDTO> indexList = Lists.newLinkedList();
            if (type.equals(ConceptTypeEnum.LisPack.getKey()) && StringUtil.isNotBlank(item.getHisDetailName())) {
                if (synonymsSubNameMap.containsKey(item.getHisDetailName())) {
                    indexList = synonymsSubNameMap.get(item.getHisDetailName());
                }
            } else {
                if (synonymsNameMap.containsKey(item.getHisName())) {
                    indexList = synonymsNameMap.get(item.getHisName());
                }
            }

            if (ListUtil.isEmpty(indexList)) {
                synonymsList.add(item);
                continue;
            }

            for (IndexBatchDTO index : indexList) {
                MappingConfigWrapper synonymsItem = new MappingConfigWrapper();
                BeanUtils.copyProperties(item, synonymsItem);
                synonymsItem.setUniqueName(index.getName());
                synonymsItem.setConceptId(index.getId());
                synonymsItem.setCode(index.getCode());
                synonymsItem.setIsMatch(1);
                synonymsItem.setSource(MatchSourceEnum.SynonymsWord.getKey());
                synonymsList.add(synonymsItem);
            }
        }

        //todo 编码匹配
        if (type.equals(ConceptTypeEnum.Disease.getKey())
                || type.equals(ConceptTypeEnum.Operation.getKey())
                || type.equals(ConceptTypeEnum.Tcmdisease.getKey())
                || type.equals(ConceptTypeEnum.Tcmsyndrome.getKey())) {
            List<String> codes = synonymsList.stream()
                    .filter(i -> (i.getIsMatch() == null || i.getIsMatch().equals(0))
                            && StringUtil.isNotBlank(i.getCode()))
                    .map(MappingConfigWrapper::getCode)
                    .distinct()
                    .collect(Collectors.toList());
            if (ListUtil.isEmpty(codes)) {
                codeList = BeanUtil.listCopyTo(synonymsList, MappingConfigWrapper.class);
            } else {
                conceptVO.setSource(MatchSourceEnum.Code.getKey());
                conceptVO.setType(type);
                conceptVO.setNames(codes);
                respDTO = cdssCoreClient.getConceptNames(conceptVO);
                RespDTOUtil.respNGDealCover(respDTO, "标准术语校验失败");
                Map<String, List<IndexBatchDTO>> codeMap = new HashMap<>();
                if (ListUtil.isNotEmpty(respDTO.data)) {
                    codeMap = respDTO.data.stream().collect(Collectors.groupingBy(IndexBatchDTO::getCode));
                }
                for (MappingConfigWrapper item : synonymsList) {
                    if (item.getIsMatch() != null && item.getIsMatch().equals(1)) {
                        codeList.add(item);
                        continue;
                    }
                    if (StringUtil.isNotBlank(item.getCode()) && codeMap.containsKey(item.getCode())) {
                        List<IndexBatchDTO> indexList = codeMap.get(item.getCode());
                        for (IndexBatchDTO index : indexList) {
                            MappingConfigWrapper codeItem = new MappingConfigWrapper();
                            BeanUtils.copyProperties(item, codeItem);
                            codeItem.setUniqueName(index.getName());
                            codeItem.setConceptId(index.getId());
                            codeItem.setCode(index.getCode());
                            codeItem.setIsMatch(1);
                            codeItem.setSource(MatchSourceEnum.Code.getKey());
                            codeList.add(codeItem);
                        }
                    } else {
                        codeList.add(item);
                        continue;
                    }
                }
            }
        } else {
            codeList = BeanUtil.listCopyTo(synonymsList, MappingConfigWrapper.class);
        }

        //todo 历史匹配
        List<String> hisNames = codeList.stream()
                .filter(i -> i.getIsMatch() == null || i.getIsMatch().equals(0))
                .map(MappingConfigWrapper::getHisName)
                .distinct()
                .collect(Collectors.toList());
        if (ListUtil.isNotEmpty(hisNames)) {
            Map<String, Map<String, Map<String, List<String>>>> configMap = groupByHisNameWithName(hisNames, type, null);

            for (MappingConfigWrapper item : codeList) {
                if (item.getIsMatch() != null && item.getIsMatch().equals(1)) {
                    historyList.add(item);
                    continue;
                }
                if (configMap.containsKey(item.getHisName())) {
                    Map<String, Map<String, List<String>>> subMap = configMap.get(item.getHisName());
                    if (subMap != null && subMap.size() > 0) {
                        Map<String, List<String>> thirdMap = subMap.get(item.getHisDetailName());
                        if (thirdMap == null || thirdMap.size() == 0) {
                            historyList.add(item);
                            continue;
                        }
                        for (Map.Entry<String, List<String>> thirdEntry : thirdMap.entrySet()) {
                            MappingConfigWrapper historyItem = new MappingConfigWrapper();
                            BeanUtils.copyProperties(item, historyItem);
                            historyItem.setUniqueName(thirdEntry.getKey());
                            historyItem.setIsMatch(1);
                            historyItem.setSource(MatchSourceEnum.History.getKey());
                            if (type.equals(ConceptTypeEnum.Drug.getKey())) {
                                List<String> forms = thirdEntry.getValue();
                                if (ListUtil.isEmpty(forms)) {
                                    historyList.add(historyItem);
                                } else {
                                    for (String form : forms) {
                                        MappingConfigWrapper formItem = new MappingConfigWrapper();
                                        BeanUtil.copyProperties(historyItem, formItem);
                                        formItem.setForm(form);
                                        historyList.add(formItem);
                                    }
                                }
                            } else {
                                historyList.add(historyItem);
                            }
                        }
                    }
                } else {
                    historyList.add(item);
                }
            }
        } else {
            historyList = BeanUtil.listCopyTo(codeList, MappingConfigWrapper.class);
        }

        //药品关联标准词、同义词匹配数据增加历史数据剂型
        if (type.equals(ConceptTypeEnum.Drug.getKey())) {
            List<String> hisDrugNames = historyList.stream()
                    .filter(i -> i.getIsMatch() != null
                            && i.getIsMatch().equals(1)
                            && i.getSource() != null
                            && (i.getSource().equals(MatchSourceEnum.StandWord.getKey())))
                    .map(MappingConfigWrapper::getHisName)
                    .distinct()
                    .collect(Collectors.toList());
            hisDrugNames.addAll(historyList.stream()
                    .filter(i -> i.getIsMatch() != null
                            && i.getIsMatch().equals(1)
                            && i.getSource() != null
                            && i.getSource().equals(MatchSourceEnum.SynonymsWord.getKey()))
                    .map(MappingConfigWrapper::getUniqueName)
                    .distinct()
                    .collect(Collectors.toList()));
            if (ListUtil.isNotEmpty(hisDrugNames)) {
                Map<String, Map<String, Map<String, List<String>>>> configMap = groupByHisNameWithName(hisDrugNames, type, null);
                for (MappingConfigWrapper item : historyList) {
                    if (item.getIsMatch() != null && item.getIsMatch().equals(1)
                            && item.getSource() != null
                            && (item.getSource().equals(MatchSourceEnum.StandWord.getKey())
                            || item.getSource().equals(MatchSourceEnum.SynonymsWord.getKey()))) {
                        if ((item.getSource().equals(MatchSourceEnum.StandWord.getKey())
                                && configMap.containsKey(item.getHisName()))
                                || (item.getSource().equals(MatchSourceEnum.SynonymsWord.getKey())
                                && configMap.containsKey(item.getUniqueName()))) {
                            Map<String, Map<String, List<String>>> subMap
                                    = configMap.get(item.getSource().equals(MatchSourceEnum.StandWord.getKey())
                                    ? item.getHisName()
                                    : item.getUniqueName());
                            if (subMap != null && subMap.size() > 0) {
                                Map<String, List<String>> thirdMap = subMap.get(item.getHisDetailName());
                                if (thirdMap == null || thirdMap.size() == 0) {
                                    drugList.add(item);
                                    continue;
                                }
                                List<String> forms = thirdMap.get(item.getUniqueName());
                                if (ListUtil.isEmpty(forms)) {
                                    drugList.add(item);
                                } else {
                                    for (String form : forms) {
                                        MappingConfigWrapper formItem = new MappingConfigWrapper();
                                        BeanUtil.copyProperties(item, formItem);
                                        formItem.setForm(form);
                                        drugList.add(formItem);
                                    }
                                }
                            }
                        } else {
                            drugList.add(item);
                        }
                    } else {
                        drugList.add(item);
                    }
                }
            } else {
                drugList = BeanUtil.listCopyTo(historyList, MappingConfigWrapper.class);
            }
        } else {
            drugList = BeanUtil.listCopyTo(historyList, MappingConfigWrapper.class);
        }


        retList = BeanUtil.listCopyTo(drugList, MappingConfigWrapper.class);
        for (MappingConfigWrapper item : retList) {
            if (item.getIsMatch() == null) {
                item.setIsMatch(0);
            }
        }

        //添加编码
        retList = addCodes(retList, type);
        retList = retList.stream().distinct().collect(Collectors.toList());
        return retList;
    }

    /**
     * 数据导入
     *
     * @param file
     * @param hospitalId
     * @param type
     */
    public Boolean importExcel(MultipartFile file, Long hospitalId, Integer type, String userId) {
        if (hospitalId == null) {
            hospitalId = Long.valueOf(SysUserUtils.getCurrentHospitalID());
        }
        if (StringUtil.isBlank(userId)) {
            userId = SysUserUtils.getCurrentPrincipleID();
        }
        if (StringUtil.isBlank(userId)) {
            userId = "0";
        }
        List<MappingConfigWrapper> originList = readImportData(file, type, 2);
        if (ListUtil.isEmpty(originList)) {
            throw new CommonException(CommonErrorCode.PARAM_IS_NULL, "校验失败，导入数据不能为空");
        }
        Boolean data = importDataProcess(originList, hospitalId, type, userId);
        return data;
    }

    /**
     * 导入数据处理
     *
     * @param originList
     * @param hospitalId
     * @param type
     */
    public Boolean importDataProcess(List<MappingConfigWrapper> originList, Long hospitalId, Integer type, String userId) {
        List<MappingConfigWrapper> tempList = Lists.newLinkedList();

        Date now = DateUtil.now();

        //过滤空数据，保留重复数据，方便计行
        //去除空格、回车、换行符、制表符
        originList = originList.stream()
                .filter(MappingConfigWrapper::nonNull)
                .collect(Collectors.toList());
        if (ListUtil.isEmpty(originList)) {
            throw new CommonException(CommonErrorCode.PARAM_IS_NULL, "校验失败，导入数据不能为空");
        }
        List<Integer> emptyNumList = Lists.newLinkedList();
        Integer rowId = 1;
        for (MappingConfigWrapper item : originList) {
            if (StringUtil.isBlank(item.getHisName())) {
                if (type.equals(ConceptTypeEnum.Drug.getKey())) {
                    emptyNumList.add(rowId + 2);
                } else {
                    emptyNumList.add(rowId + 1);
                }
            } else {
                item.setHisName(pattern.matcher(item.getHisName()).replaceAll(""));
            }
            if (StringUtil.isNotBlank(item.getHisCode())) {
                item.setHisCode(pattern.matcher(item.getHisCode()).replaceAll(""));
            } else {
                item.setHisCode("");
            }
            if (StringUtil.isNotBlank(item.getHisDetailName())) {
                item.setHisDetailName(pattern.matcher(item.getHisDetailName()).replaceAll(""));
            } else {
                item.setHisDetailName("");
            }
            if (StringUtil.isNotBlank(item.getUniqueName())) {
                item.setUniqueName(pattern.matcher(item.getUniqueName()).replaceAll(""));
                item.setIsMatch(1);
            } else {
                item.setUniqueName("");
                item.setIsMatch(0);
            }
            if (StringUtil.isNotBlank(item.getForm())) {
                item.setForm(pattern.matcher(item.getForm()).replaceAll(""));
            } else {
                item.setForm("");
            }
            if (StringUtil.isNotBlank(item.getCode())) {
                item.setCode(pattern.matcher(item.getCode()).replaceAll(""));
            } else {
                item.setCode("");
            }
            rowId++;
        }

        //医院术语名称不允许为空
        if (ListUtil.isNotEmpty(emptyNumList)) {
            throw new CommonException(CommonErrorCode.PARAM_IS_NULL, "以下行数（不计入空行）存在不完整数据："
                    + emptyNumList.stream().map(Object::toString)
                    .collect(Collectors.joining("、"))
                    + "。导入取消，请修改后再试。\n");
        }

        List<String> hisNames = originList
                .stream()
                .map(i -> i.getHisName())
                .collect(Collectors.toList());

        originList = originList.stream().distinct().collect(Collectors.toList());
        List<MappingConfigWrapper> matchList
                = originList.stream().filter(i -> StringUtil.isNotBlank(i.getUniqueName())).collect(Collectors.toList());
        Map<String, List<MappingConfigWrapper>> matchMap
                = matchList.stream().collect(Collectors.groupingBy(i -> i.getHisName() + "_"
                + i.getHisCode() + "_"
                + i.getHisDetailName()));
        Map<String, List<MappingConfigWrapper>> fullMatchMap = matchList.stream()
                .collect(Collectors.groupingBy(i -> i.getHisName() + "_"
                        + i.getHisCode() + "_"
                        + i.getHisDetailName() + "_"
                        + i.getUniqueName() + "_"
                        + i.getForm()));
        List<MappingConfigWrapper> unMatchList
                = originList.stream().filter(i -> StringUtil.isBlank(i.getUniqueName())).collect(Collectors.toList());
        Map<String, List<MappingConfigWrapper>> unMatchMap
                = unMatchList.stream().collect(Collectors.groupingBy(i -> i.getHisName() + "_"
                + i.getHisCode() + "_"
                + i.getHisDetailName()));
        Iterator<MappingConfigWrapper> it = unMatchList.iterator();
        while (it.hasNext()) {
            MappingConfigWrapper item = it.next();
            String matchKey = item.getHisName() + "_"
                    + item.getHisCode() + "_"
                    + item.getHisDetailName();
            if (matchMap.containsKey(matchKey)) {
                it.remove();
            }
        }

        // 验证数据是否已存在，已存在的先删除
        // 没id的删除重新插入，有id的更新
        synchronized (this) {
            List<Long> deleteIds = Lists.newLinkedList();
            List<MappingConfig> existList = this.list(new QueryWrapper<MappingConfig>()
                    .eq("is_deleted", IsDeleteEnum.N.getKey())
                    .eq("hospital_id", hospitalId)
                    .eq("type", type)
                    .in("his_name", hisNames));

            if (ListUtil.isNotEmpty(existList)) {
                List<MappingConfigWrapper> existWrapperList = BeanUtil.listCopyTo(existList, MappingConfigWrapper.class);
                existWrapperList = addNames(existWrapperList);
                for (MappingConfigWrapper item : existWrapperList) {
                    if (StringUtil.isBlank(item.getHisDetailName())) {
                        item.setHisDetailName("");
                    }
                    if (StringUtil.isBlank(item.getUniqueName())) {
                        item.getUniqueName();
                    }
                    if (StringUtil.isBlank(item.getForm())) {
                        item.setForm("");
                    }
                    if (StringUtil.isBlank(item.getHisCode())) {
                        item.setHisCode("");
                    }
                }

                List<MappingConfigWrapper> db_matchList
                        = existWrapperList.stream().filter(i -> i.getIsMatch().equals(1)).collect(Collectors.toList());
                Map<String, List<MappingConfigWrapper>> db_matchMap
                        = db_matchList.stream().collect(Collectors.groupingBy(i -> i.getHisName() + "_"
                        + i.getHisCode() + "_"
                        + i.getHisDetailName()));
                List<MappingConfigWrapper> db_unMatchList
                        = existWrapperList.stream().filter(i -> i.getIsMatch().equals(0)).collect(Collectors.toList());
                Map<String, List<MappingConfigWrapper>> db_unMatchMap
                        = db_unMatchList.stream().collect(Collectors.groupingBy(i -> i.getHisName() + "_"
                        + i.getHisCode() + "_"
                        + i.getHisDetailName()));

                for (MappingConfigWrapper item : db_matchList) {
                    String matchKey = item.getHisName() + "_"
                            + item.getHisCode() + "_"
                            + item.getHisDetailName() + "_"
                            + item.getUniqueName() + "_"
                            + item.getForm();
                    if (fullMatchMap.containsKey(matchKey)) {
                        deleteIds.add(item.getId());
                    }
                }
                for (MappingConfigWrapper item : db_unMatchList) {
                    String matchKey = item.getHisName() + "_"
                            + item.getHisCode() + "_"
                            + item.getHisDetailName();
                    if (matchMap.containsKey(matchKey)
                            || db_matchMap.containsKey(matchKey)
                            || unMatchMap.containsKey(matchKey)) {
                        deleteIds.add(item.getId());
                    }
                }

                //数据库中已存在匹配项，去除导入文件中未匹配项
                it = unMatchList.iterator();
                while (it.hasNext()) {
                    MappingConfigWrapper item = it.next();
                    String matchKey = item.getHisName() + "_"
                            + item.getHisCode() + "_"
                            + item.getHisDetailName();
                    if (db_matchMap.containsKey(matchKey)) {
                        it.remove();
                    }
                }
            }

            //过滤后的导入结果集
            tempList.addAll(matchList);
            tempList.addAll(unMatchList);

            //标准术语校验
            ConceptVO conceptVO = new ConceptVO();
            conceptVO.setSource(MatchSourceEnum.StandWord.getKey());
            if (type.equals(ConceptTypeEnum.LisPack.getKey())) {
                conceptVO.setType(ConceptTypeEnum.LisPack.getKey());
                conceptVO.setNames(tempList.stream()
                        .filter(i -> StringUtil.isBlank(i.getHisDetailName())
                                && StringUtil.isNotBlank(i.getUniqueName()))
                        .map(MappingConfigWrapper::getUniqueName)
                        .distinct()
                        .collect(Collectors.toList()));
                RespDTO<List<IndexBatchDTO>> respDTO = cdssCoreClient.getConceptNames(conceptVO);
                RespDTOUtil.respNGDealCover(respDTO, "标准术语校验失败");
                Map<String, List<Long>> packMap = respDTO.data.stream()
                        .collect(Collectors.groupingBy(IndexBatchDTO::getName,
                                Collectors.mapping(IndexBatchDTO::getId, Collectors.toList())));
                conceptVO.setType(ConceptTypeEnum.Lis.getKey());
                conceptVO.setNames(tempList.stream()
                        .filter(i -> StringUtil.isNotBlank(i.getHisDetailName())
                                && StringUtil.isNotBlank(i.getUniqueName()))
                        .map(MappingConfigWrapper::getUniqueName)
                        .distinct()
                        .collect(Collectors.toList()));
                respDTO = cdssCoreClient.getConceptNames(conceptVO);
                RespDTOUtil.respNGDealCover(respDTO, "标准术语校验失败");
                Map<String, List<Long>> lisMap = respDTO.data.stream()
                        .collect(Collectors.groupingBy(IndexBatchDTO::getName,
                                Collectors.mapping(IndexBatchDTO::getId, Collectors.toList())));

                for (MappingConfigWrapper item : tempList) {
                    if (StringUtil.isBlank(item.getUniqueName())) {
                        continue;
                    }
                    if (StringUtil.isBlank(item.getHisDetailName())) {
                        if (packMap.containsKey(item.getUniqueName())) {
                            item.setConceptId(packMap.get(item.getUniqueName()).get(0));
                            item.setIsMatch(1);
                        } else {
                            item.setIsMatch(0);
                        }
                    } else {
                        if (lisMap.containsKey(item.getUniqueName())) {
                            item.setConceptId(lisMap.get(item.getUniqueName()).get(0));
                            item.setIsMatch(1);
                        } else {
                            item.setIsMatch(0);
                        }
                    }

                }
            } else {
                conceptVO.setType(type);
                conceptVO.setNames(tempList.stream()
                        .filter(i -> StringUtil.isNotBlank(i.getUniqueName()))
                        .map(MappingConfigWrapper::getUniqueName)
                        .distinct()
                        .collect(Collectors.toList()));
                RespDTO<List<IndexBatchDTO>> respDTO = cdssCoreClient.getConceptNames(conceptVO);
                if (RespDTOUtil.respIsOK(respDTO)) {
                    Map<String, List<Long>> map = respDTO.data.stream()
                            .collect(Collectors.groupingBy(IndexBatchDTO::getName,
                                    Collectors.mapping(IndexBatchDTO::getId, Collectors.toList())));
                    for (MappingConfigWrapper item : tempList) {
                        if (StringUtil.isBlank(item.getUniqueName())) {
                            continue;
                        }
                        if (map.containsKey(item.getUniqueName())) {
                            item.setConceptId(map.get(item.getUniqueName()).get(0));
                            item.setIsMatch(1);
                        } else {
                            item.setIsMatch(0);
                        }
                    }
                }
            }

            //剂型校验
            if (type.equals(ConceptTypeEnum.Drug.getKey())) {
                conceptVO.setType(ConceptTypeEnum.Form.getKey());
                conceptVO.setNames(tempList.stream()
                        .filter(i -> StringUtil.isNotBlank(i.getForm()))
                        .map(MappingConfigWrapper::getForm)
                        .distinct()
                        .collect(Collectors.toList()));
                RespDTO<List<IndexBatchDTO>> respDTO = cdssCoreClient.getConceptNames(conceptVO);
                if (RespDTOUtil.respIsOK(respDTO)) {
                    Map<String, List<Long>> formMap = respDTO.data.stream()
                            .collect(Collectors.groupingBy(IndexBatchDTO::getName,
                                    Collectors.mapping(IndexBatchDTO::getId, Collectors.toList())));
                    for (MappingConfigWrapper item : tempList) {
                        if (StringUtil.isBlank(item.getForm())) {
                            continue;
                        }
                        if (formMap.containsKey(item.getForm())) {
                            item.setFormConceptId(formMap.get(item.getForm()).get(0));
                        }
                    }
                }
            }

            //重复数据过滤
            tempList = tempList
                    .stream()
                    .distinct()
                    .collect(Collectors.toList());

            for (MappingConfigWrapper item : tempList) {
                item.setHospitalId(hospitalId);
                item.setType(type);
                item.setCreator(userId);
                item.setGmtCreate(now);
                item.setModifier(userId);
                item.setGmtModified(now);
            }

            //删除已存在映射关系
            IdListVO idListVO = new IdListVO();
            idListVO.setIds(deleteIds);
            deleteRecords(idListVO);
            List<MappingConfig> saveList = BeanUtil.listCopyTo(tempList, MappingConfig.class);
            saveList = saveList.stream().distinct().collect(Collectors.toList());
            if (ListUtil.isNotEmpty(saveList)) {
                mappingConfigService.saveOrUpdateBatch(saveList);
            }
        }
        return true;
    }

    /**
     * 删除记录-单条
     *
     * @param idVO
     * @return
     */
    public Boolean deleteRecord(IdVO idVO) {
        UpdateWrapper<MappingConfig> updateWrapper = new UpdateWrapper<>();
        updateWrapper.eq("id", idVO.getId())
                .set("is_deleted", IsDeleteEnum.Y.getKey());
        mappingConfigService.removeById(idVO.getId());
        return true;
    }

    /**
     * 删除记录-批量
     *
     * @param idListVO
     * @return
     */
    public Boolean deleteRecords(IdListVO idListVO) {
        if (ListUtil.isEmpty(idListVO.getIds())) {
            return false;
        }
        UpdateWrapper<MappingConfig> updateWrapper = new UpdateWrapper<>();
        updateWrapper.in("id", idListVO.getIds())
                .set("is_deleted", IsDeleteEnum.Y.getKey());
        mappingConfigService.removeByIds(idListVO.getIds());
        return true;
    }

    /**
     * 获取记录-单条
     *
     * @param idVO
     * @return
     */
    public MappingConfigWrapper getRecord(IdVO idVO) {
        MappingConfig item = this.getById(idVO.getId());
        MappingConfigWrapper itemWrapper = new MappingConfigWrapper();
        BeanUtils.copyProperties(item, itemWrapper);
        List<MappingConfigWrapper> retList = Lists.newArrayList();
        retList.add(itemWrapper);
        retList = addNames(retList);

        return retList.get(0);
    }

    /**
     * 查询已关联关系
     *
     * @param mappingConfigVO
     * @return
     */
    public List<MappingConfigWrapper> getRelatedMapping(MappingConfigVO mappingConfigVO) {
        List<MappingConfigWrapper> retList = Lists.newLinkedList();
        if (mappingConfigVO.getHospitalId() == null) {
            mappingConfigVO.setHospitalId(Long.valueOf(SysUserUtils.getCurrentHospitalID()));
        }
        QueryWrapper<MappingConfig> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("is_deleted", IsDeleteEnum.N.getKey())
                .eq("hospital_id", mappingConfigVO.getHospitalId())
                .eq("type", mappingConfigVO.getType())
                .eq("his_name", mappingConfigVO.getHisName())
                .eq("is_match", 1);
        if (StringUtil.isNotBlank(mappingConfigVO.getHisDetailName())) {
            queryWrapper.eq("his_detail_name", mappingConfigVO.getHisDetailName());
        } else {
            queryWrapper.and(i -> i.isNull("his_detail_name")
                    .or()
                    .eq("his_detail_name", ""));
        }

        List<MappingConfig> list = this.list(queryWrapper);

        if (ListUtil.isEmpty(list)) {
            return retList;
        }

        retList = BeanUtil.listCopyTo(list, MappingConfigWrapper.class);

        retList = addNames(retList);
        return retList;
    }

    /**
     * 判断是否已存在
     *
     * @param mappingConfig
     * @return
     */
    public Boolean isExistRecord(MappingConfig mappingConfig) {
        if (mappingConfig.getType() == null) {
            throw new CommonException(CommonErrorCode.PARAM_IS_NULL, "请输入请输入术语类型：1-化验、3-辅检、4-诊断、5-药品、6-手术和操作、7-科室、8-输血、10-量表、11-护理、12-中医诊断、13-中医证候、14-麻醉");
        }
        if (mappingConfig.getHospitalId() == null) {
            mappingConfig.setHospitalId(Long.valueOf(SysUserUtils.getCurrentHospitalID()));
        }
        QueryWrapper<MappingConfig> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("is_deleted", IsDeleteEnum.N.getKey())
                .eq("hospital_id", mappingConfig.getHospitalId())
                .eq("type", mappingConfig.getType())
                .eq("his_name", mappingConfig.getHisName())
                .eq("concept_id", mappingConfig.getConceptId());
        if (StringUtil.isBlank(mappingConfig.getHisDetailName())) {
            queryWrapper.and(i -> i.isNull("his_detail_name")
                    .or()
                    .eq("his_detail_name", ""));
        } else {
            queryWrapper
                    .eq("his_detail_name", mappingConfig.getHisDetailName());
        }
        if (StringUtil.isBlank(mappingConfig.getHisCode())) {
            queryWrapper.and(i -> i.isNull("his_code")
                    .or()
                    .eq("his_code", ""));
        } else {
            queryWrapper
                    .eq("his_code", mappingConfig.getHisCode());
        }
        if (mappingConfig.getFormConceptId() != null) {
            queryWrapper.eq("form_concept_id", mappingConfig.getFormConceptId());
        } else {
            queryWrapper.isNull("form_concept_id");
        }
        if (mappingConfig.getFormConceptId() != null) {
            queryWrapper.eq("form_concept_id", mappingConfig.getFormConceptId());
        }
        MappingConfig oldRecord = this.getOne(queryWrapper, false);
        if (mappingConfig.getId() == null
                && oldRecord != null) {
            throw new CommonException(CommonErrorCode.IS_EXISTS, "该条关联已存在，无法保存");
        }
        if (mappingConfig.getId() != null
                && oldRecord != null
                && !mappingConfig.getId().equals(oldRecord.getId())) {
            throw new CommonException(CommonErrorCode.IS_EXISTS, "该条关联已存在，无法保存");
        }
        return false;
    }

    /**
     * 保存记录-单条
     *
     * @param mappingConfig
     * @return
     */
    public Boolean saveOrUpdateRecord(MappingConfig mappingConfig) {
        if (mappingConfig.getType() == null) {
            throw new CommonException(CommonErrorCode.PARAM_IS_NULL, "请输入请输入术语类型：1-化验、3-辅检、4-诊断、5-药品、6-手术和操作、7-科室、8-输血、10-量表、11-护理、12-中医诊断、13-中医证候、14-麻醉");
        }
        if (mappingConfig.getConceptId() == null) {
            throw new CommonException(CommonErrorCode.PARAM_IS_NULL, "标准术语不允许为空，请输入标准术语id");
        }
        if (mappingConfig.getHospitalId() == null) {
            mappingConfig.setHospitalId(Long.valueOf(SysUserUtils.getCurrentHospitalID()));
        }
        String userId = "";
        if (StringUtil.isBlank(mappingConfig.getModifier())) {
            userId = SysUserUtils.getCurrentPrincipleID();
        }
        if (StringUtil.isBlank(userId)) {
            userId = "0";
        }
        Date now = DateUtil.now();
        mappingConfig.setModifier(userId);
        mappingConfig.setGmtModified(now);
        QueryWrapper<MappingConfig> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("is_deleted", IsDeleteEnum.N.getKey())
                .eq("hospital_id", mappingConfig.getHospitalId())
                .eq("type", mappingConfig.getType())
                .eq("his_name", mappingConfig.getHisName())
                .eq("concept_id", mappingConfig.getConceptId());
        if (StringUtil.isBlank(mappingConfig.getHisDetailName())) {
            queryWrapper.and(i -> i.isNull("his_detail_name")
                    .or()
                    .eq("his_detail_name", ""));
        } else {
            queryWrapper.eq("his_detail_name", mappingConfig.getHisDetailName());
        }
        if (StringUtil.isBlank(mappingConfig.getHisCode())) {
            queryWrapper.and(i -> i.isNull("his_code")
                    .or()
                    .eq("his_code", ""));
        } else {
            queryWrapper
                    .eq("his_code", mappingConfig.getHisCode());
        }
        if (mappingConfig.getFormConceptId() != null) {
            queryWrapper.eq("form_concept_id", mappingConfig.getFormConceptId());
        } else {
            queryWrapper.isNull("form_concept_id");
        }
        MappingConfig oldRecord = this.getOne(queryWrapper, false);
        if (mappingConfig.getId() == null
                && oldRecord != null) {
            throw new CommonException(CommonErrorCode.IS_EXISTS, "该条关联已存在，无法保存");
        }
        if (mappingConfig.getId() != null
                && oldRecord != null
                && !mappingConfig.getId().equals(oldRecord.getId())) {
            throw new CommonException(CommonErrorCode.IS_EXISTS, "该条关联已存在，无法保存");
        }
        //新增数据
        if (mappingConfig.getId() == null) {
            mappingConfig.setCreator(userId);
            mappingConfig.setGmtCreate(now);
        }
        if (mappingConfig.getIsDeleted() == null) {
            mappingConfig.setIsDeleted(IsDeleteEnum.N.getKey());
        }

        //删除未匹配的关系
        QueryWrapper<MappingConfig> qwUnMaptch = new QueryWrapper<>();
        qwUnMaptch.eq("is_deleted", IsDeleteEnum.N.getKey())
                .eq("hospital_id", mappingConfig.getHospitalId())
                .eq("type", mappingConfig.getType())
                .eq("his_name", mappingConfig.getHisName())
                .isNull("concept_id")
                .eq("is_match", 0);
        if (StringUtil.isBlank(mappingConfig.getHisDetailName())) {
            qwUnMaptch.and(i -> i.isNull("his_detail_name")
                    .or()
                    .eq("his_detail_name", ""));
        } else {
            qwUnMaptch.eq("his_detail_name", mappingConfig.getHisDetailName());
        }
        if (StringUtil.isBlank(mappingConfig.getHisCode())) {
            qwUnMaptch.and(i -> i.isNull("his_code")
                    .or()
                    .eq("his_code", ""));
        } else {
            qwUnMaptch
                    .eq("his_code", mappingConfig.getHisCode());
        }
        this.remove(qwUnMaptch);

        mappingConfig.setIsMatch(1);
        this.saveOrUpdate(mappingConfig);
        return true;
    }


    /**
     * 根据医院名称分组-返回id
     *
     * @param hisNames
     * @param type
     * @return
     */
    public Map<String, Map<String, Map<Long, List<Long>>>> groupByHisNameWithId(List<String> hisNames, Integer type, Long hospitalId) {
        Map<String, Map<String, Map<Long, List<Long>>>> retMap = new HashMap<>();
        QueryWrapper<MappingConfig> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("is_deleted", IsDeleteEnum.N.getKey())
                .eq("is_match", 1)
                .eq("type", type);
        if (hospitalId != null) {
            queryWrapper.eq("hospital_id", hospitalId);
        }
        if (ListUtil.isNotEmpty(hisNames)) {
            queryWrapper.in("his_name", hisNames);
        }
        List<MappingConfig> list = this.list(queryWrapper);

        if (ListUtil.isEmpty(list)) {
            return retMap;
        }

        for (MappingConfig item : list) {
            if (StringUtil.isBlank(item.getHisDetailName())) {
                item.setHisDetailName("");
            }
            item.setId(null);
            item.setHospitalId(null);
        }

        list = list.stream().distinct().collect(Collectors.toList());

        Map<String, List<MappingConfig>> hisMap = list.stream()
                .filter(i -> StringUtil.isNotBlank(i.getHisName()))
                .collect(Collectors.groupingBy(MappingConfig::getHisName));
        for (Map.Entry<String, List<MappingConfig>> entry : hisMap.entrySet()) {
            Map<String, Map<Long, List<Long>>> subMap = new HashMap<>();
            Map<String, List<MappingConfig>> detailMap = entry.getValue().stream()
                    .collect(Collectors.groupingBy(MappingConfig::getHisDetailName));
            for (Map.Entry<String, List<MappingConfig>> subEntry : detailMap.entrySet()) {
                subMap.put(subEntry.getKey(),
                        subEntry.getValue()
                                .stream()
                                .collect(Collectors.groupingBy(MappingConfig::getConceptId,
                                        Collectors.mapping(MappingConfig::getFormConceptId, Collectors.toList()))));
            }
            retMap.put(entry.getKey(), subMap);
        }
        return retMap;
    }

    /**
     * 根据医院名称分组-返回名称
     *
     * @param hisNames
     * @param type
     * @param hospitalId
     * @return
     */
    public Map<String, Map<String, Map<String, List<String>>>> groupByHisNameWithName(List<String> hisNames, Integer type, Long hospitalId) {
        Map<String, Map<String, Map<String, List<String>>>> retMap = new HashMap<>();
        Map<String, Map<String, Map<Long, List<Long>>>> idMap = groupByHisNameWithId(hisNames, type, hospitalId);

        //conceptIds
        List<Long> ids = idMap.values()
                .stream()
                .flatMap(i -> i.values().stream())
                .map(i -> i.keySet())
                .flatMap(Collection::stream)
                .filter(i -> i != null)
                .distinct()
                .collect(Collectors.toList());

        //formConceptId
        ids.addAll(idMap.values()
                .stream()
                .flatMap(i -> i.values().stream())
                .flatMap(i -> i.values().stream())
                .flatMap(Collection::stream)
                .filter(i -> i != null)
                .distinct()
                .collect(Collectors.toList()));

        ConceptVO conceptVO = new ConceptVO();
        conceptVO.setSource(-1);
        conceptVO.setType(type);
        conceptVO.setIds(ids);
        RespDTO<List<IndexBatchDTO>> respDTO = cdssCoreClient.getConceptNames(conceptVO);
        RespDTOUtil.respNGDealCover(respDTO, "标准术语校验失败");
        List<IndexBatchDTO> indexList = respDTO.data;
        if (ListUtil.isNotEmpty(indexList)) {
            Map<Long, String> idNameMap
                    = indexList.stream().collect(Collectors.toMap(IndexBatchDTO::getId, IndexBatchDTO::getName));
            for (Map.Entry<String, Map<String, Map<Long, List<Long>>>> entry : idMap.entrySet()) {
                Map<String, Map<String, List<String>>> subMap = new HashMap<>();
                for (Map.Entry<String, Map<Long, List<Long>>> subEntry : entry.getValue().entrySet()) {
                    Map<String, List<String>> thirdMap = new HashMap<>();
                    for (Map.Entry<Long, List<Long>> thirdEntry : subEntry.getValue().entrySet()) {
                        List<Long> formConceptIds = thirdEntry.getValue();
                        List<String> forms = Lists.newArrayList();
                        if (ListUtil.isNotEmpty(formConceptIds)) {
                            for (Long formConceptId : formConceptIds) {
                                if (formConceptId == null) {
                                    forms.add("");
                                } else {
                                    forms.add(idNameMap.get(formConceptId));
                                }
                            }
                            forms = forms.stream().distinct().collect(Collectors.toList());
                            thirdMap.put(idNameMap.get(thirdEntry.getKey()), forms);
                        }
                    }
                    subMap.put(subEntry.getKey(), thirdMap);
                }
                if (subMap != null && subMap.size() > 0) {
                    retMap.put(entry.getKey(), subMap);
                }
            }
        }
        return retMap;
    }


    /**
     * 根据标准名称分组-返回名称
     *
     * @param uniqueNames
     * @param type
     * @param hospitalId
     * @return uniqueName, form, hisName, hisDetailName
     */
    public Map<String, Map<String, Map<String, List<String>>>> groupByUniqueNameWithName(List<String> uniqueNames, Integer type, Long hospitalId) {
        Map<String, Map<String, Map<String, List<String>>>> retMap = new HashMap<>();

        List<Long> conceptIds = Lists.newArrayList();

        ConceptVO conceptVO = new ConceptVO();
        RespDTO<List<IndexBatchDTO>> respDTO = null;
        List<IndexBatchDTO> indexList = Lists.newArrayList();
        if (ListUtil.isNotEmpty(uniqueNames)) {
            conceptVO.setType(type);
            conceptVO.setNames(uniqueNames);
            conceptVO.setSource(MatchSourceEnum.StandWord.getKey());
            respDTO = cdssCoreClient.getConceptNames(conceptVO);
            if (RespDTOUtil.respIsOK(respDTO)) {
                indexList.addAll(respDTO.data);
            }

            if (type.equals(ConceptTypeEnum.LisPack.getKey())) {
                conceptVO.setType(ConceptTypeEnum.Lis.getKey());
                respDTO = cdssCoreClient.getConceptNames(conceptVO);
                if (RespDTOUtil.respIsOK(respDTO)) {
                    indexList.addAll(respDTO.data);
                }
            }
        }

        if (ListUtil.isNotEmpty(indexList)) {
            conceptIds = indexList.stream().map(IndexBatchDTO::getId).distinct().collect(Collectors.toList());
        }

        QueryWrapper<MappingConfig> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("is_deleted", IsDeleteEnum.N.getKey())
                .eq("is_match", 1)
                .eq("type", type);
        if (hospitalId != null) {
            queryWrapper.eq("hospital_id", hospitalId);
        }
        if (ListUtil.isNotEmpty(conceptIds)) {
            queryWrapper.in("concept_id", conceptIds);
        }
        List<MappingConfig> list = this.list(queryWrapper);

        if (ListUtil.isEmpty(list)) {
            return retMap;
        }

        List<MappingConfigWrapper> wrapperList = BeanUtil.listCopyTo(list, MappingConfigWrapper.class);
        wrapperList = addNames(wrapperList);
        wrapperList.forEach(item -> {
            if (StringUtil.isBlank(item.getForm())) {
                item.setForm("");
            }
            if (StringUtil.isBlank(item.getHisDetailName())) {
                item.setHisDetailName("");
            }
        });


        Map<String, List<MappingConfigWrapper>> map = wrapperList.stream()
                .filter(i -> StringUtil.isNotBlank(i.getUniqueName()))
                .collect(Collectors.groupingBy(MappingConfigWrapper::getUniqueName));

        for (Map.Entry<String, List<MappingConfigWrapper>> entry : map.entrySet()) {
            Map<String, Map<String, List<String>>> subMap = new HashMap<>();
            Map<String, List<MappingConfigWrapper>> formMap
                    = entry.getValue().stream().collect(Collectors.groupingBy(MappingConfigWrapper::getForm));
            for (Map.Entry<String, List<MappingConfigWrapper>> subEntry : formMap.entrySet()) {
                subMap.put(subEntry.getKey(), subEntry.getValue().stream().collect(Collectors.groupingBy(MappingConfigWrapper::getHisName,
                        Collectors.mapping(MappingConfigWrapper::getHisDetailName, Collectors.toList()))));
            }
            retMap.put(entry.getKey(), subMap);
        }

        return retMap;
    }

    /**
     * 返回结果添加标准术语，药品剂型
     *
     * @param list
     * @return
     */
    public List<MappingConfigWrapper> addNames(List<MappingConfigWrapper> list) {
        if (ListUtil.isEmpty(list)) {
            return list;
        }

        List<Long> ids = Lists.newArrayList();
        ids.addAll(list.stream()
                .filter(i -> i.getConceptId() != null)
                .map(MappingConfigWrapper::getConceptId)
                .collect(Collectors.toList()));
        ids.addAll(list.stream()
                .filter(i -> i.getFormConceptId() != null)
                .map(MappingConfigWrapper::getFormConceptId)
                .collect(Collectors.toList()));

        ConceptVO conceptVO = new ConceptVO();
        conceptVO.setSource(-1);
        conceptVO.setIds(ids);
        RespDTO<List<IndexBatchDTO>> respDTO = cdssCoreClient.getConceptNames(conceptVO);
        if (RespDTOUtil.respIsOK(respDTO)) {
            Map<Long, IndexBatchDTO> conceptMap
                    = respDTO.data.stream().collect(Collectors.toMap(IndexBatchDTO::getId, v -> v));
            for (MappingConfigWrapper record : list) {
                if (record.getConceptId() != null && conceptMap.containsKey(record.getConceptId())) {
                    record.setUniqueName(conceptMap.get(record.getConceptId()).getName());
                    record.setCode(conceptMap.get(record.getConceptId()).getCode());
                }
                if (record.getFormConceptId() != null && conceptMap.containsKey(record.getFormConceptId())) {
                    record.setForm(conceptMap.get(record.getFormConceptId()).getName());
                }
            }
        }
        return list;
    }


    /**
     * 返回结果添加编码
     *
     * @param list
     * @return
     */
    public List<MappingConfigWrapper> addCodes(List<MappingConfigWrapper> list, Integer type) {
        List<MappingConfigWrapper> retList = Lists.newLinkedList();
        if (ListUtil.isEmpty(list)) {
            return list;
        }

        if (!(type.equals(ConceptTypeEnum.Operation.getKey())
                || type.equals(ConceptTypeEnum.Disease.getKey())
                || type.equals(ConceptTypeEnum.Tcmdisease.getKey())
                || type.equals(ConceptTypeEnum.Tcmsyndrome.getKey()))) {
            return list;
        }

        List<String> names = list.stream()
                .filter(i -> StringUtil.isNotBlank(i.getUniqueName()))
                .map(MappingConfigWrapper::getUniqueName)
                .collect(Collectors.toList());

        ConceptVO conceptVO = new ConceptVO();
        conceptVO.setSource(MatchSourceEnum.StandWord.getKey());
        conceptVO.setType(type);
        conceptVO.setNames(names);
        RespDTO<List<IndexBatchDTO>> respDTO = cdssCoreClient.getConceptNames(conceptVO);
        if (RespDTOUtil.respIsOK(respDTO)) {
            Map<String, List<String>> conceptMap
                    = respDTO.data.stream().collect(Collectors.groupingBy(IndexBatchDTO::getName,
                    Collectors.mapping(IndexBatchDTO::getCode, Collectors.toList())));
            for (MappingConfigWrapper record : list) {
                if (StringUtil.isNotBlank(record.getUniqueName())
                        && conceptMap.containsKey(record.getUniqueName())) {
                    for (String code : conceptMap.get(record.getUniqueName())) {
                        MappingConfigWrapper tempRecord = new MappingConfigWrapper();
                        BeanUtils.copyProperties(record, tempRecord);
                        tempRecord.setCode(code);
                        retList.add(tempRecord);
                    }
                } else {
                    retList.add(record);
                }
            }
        }
        return retList;
    }


    /**
     * 医院术语检索
     *
     * @param retrievalVO
     * @return
     */
    public List<String> getHisNames(RetrievalVO retrievalVO) {
        List<MappingConfig> records = this.hisNameIndex(retrievalVO);
        List<String> hisNames
                = records.stream().map(MappingConfig::getHisName).distinct().collect(Collectors.toList());
        return hisNames;
    }

    /**
     * 医院术语-细项检索
     *
     * @param retrievalVO
     * @return
     */
    public List<LisDetailDTO> getHisDetailNames(RetrievalVO retrievalVO) {
        retrievalVO.setType(ConceptTypeEnum.LisPack.getKey());
        List<LisDetailDTO> lisDetailNames = new ArrayList<>();
        List<MappingConfig> records = this.hisDetailNameIndex(retrievalVO);
        if (ListUtil.isNotEmpty(records)) {
            List<Long> conceptIds = records.stream()
                    .filter(i -> i.getConceptId() != null)
                    .map(MappingConfig::getConceptId).distinct().collect(Collectors.toList());
            ConceptVO conceptVO = new ConceptVO();
            conceptVO.setSource(-1);
            conceptVO.setIds(conceptIds);
            conceptVO.setType(ConceptTypeEnum.Lis.getKey());
            RespDTO<List<IndexBatchDTO>> indexRespDTO = cdssCoreClient.getConceptNames(conceptVO);
            if (RespDTOUtil.respIsOK(indexRespDTO)) {
                List<String> names = indexRespDTO.data.stream().map(IndexBatchDTO::getName).distinct().collect(Collectors.toList());
                Map<Long, List<String>> indexMap = indexRespDTO.data.stream().collect(Collectors.groupingBy(IndexBatchDTO::getId,
                        Collectors.mapping(IndexBatchDTO::getName, Collectors.toList())));
                KllisDetailVO kllisDetailVO = new KllisDetailVO();
                kllisDetailVO.setNames(names);
                RespDTO<Map<String, KllisDetailDTO>> respDTO = cdssCoreClient.getLisDetaisByNames(kllisDetailVO);
                Map<String, KllisDetailDTO> map = new HashMap<>();
                if (null != respDTO && null != respDTO.data) {
                    map = respDTO.data;
                }
                for (MappingConfig record : records) {
                    LisDetailDTO lisDetailDTO = new LisDetailDTO();
                    lisDetailDTO.setName(record.getHisName());
                    lisDetailDTO.setUniqueName(record.getHisDetailName());
                    if (null != record.getConceptId()
                            && indexMap.containsKey(record.getConceptId())
                            && (null != map.get(indexMap.get(record.getConceptId()).get(0)))) {
                        KllisDetailDTO kllisDetailDTO = map.get(indexMap.get(record.getConceptId()).get(0));
                        lisDetailDTO.setMaxValue(kllisDetailDTO.getMaxValue());
                        lisDetailDTO.setMinValue(kllisDetailDTO.getMinValue());
                        lisDetailDTO.setUnits(kllisDetailDTO.getUnit());
                        lisDetailDTO.setRange(kllisDetailDTO.getType());
                    }
                    lisDetailNames.add(lisDetailDTO);
                }
            }
        }
        return lisDetailNames;
    }
}